// *************************************************************************************
// SmVR_COBJLoader.cpp
//  Authors: Marcio Serolli Pinho - pinho@inf.pucrs.br
//           Mauro Csar Charo dos Santos - maurocharao@inf.pucrs.br
//  Homepages: http://www.smallvr.org
//             http://grv.inf.pucrs.br
//             http://www.inf.pucrs.br/~pinho
//  Porto Alegre, Brazil, 2006.
// *************************************************************************************
#include "SmVR_COBJLoader.h"
#ifdef WIN32
#include<windows.h>
#endif
#include<stdio.h>
#include<math.h>

void SmVR_COBJLoader::extract_file_path(char *nome_arq, char *buf) {
  int tam = 0;
  char *sub_str = strrchr(nome_arq, '/');
  if(sub_str == NULL || strrchr(nome_arq, '\\')> sub_str)
    sub_str = strrchr(nome_arq, '\\');
  if(sub_str != NULL) {
    tam = sub_str - nome_arq + 1;
    memcpy(buf, nome_arq, tam);
  }
  buf[tam] = '\0';
}
// ***************************************************
SmVR_COBJLoader::SmVR_COBJLoader() {
  sprintf(Name, "");
  useRAPID = 0;
}
// ***************************************************
SmVR_COBJLoader::SmVR_COBJLoader(int rapid) {
  sprintf(Name, "");
  useRAPID = rapid;
}
// ***************************************************
SmVR_COBJLoader::~SmVR_COBJLoader() {
  FreeObjectData();
}
// *******************************************************
// Clear all the memory allocated for the object
// *******************************************************
void SmVR_COBJLoader::FreeObjectData() {
  int i;
  for(i = 0; i < Vertices.size(); i++)
    delete Vertices[i];
  Vertices.clear();
  for(i = 0; i < Faces.size(); i++)
    delete Faces[i];
  Faces.clear();
  for(i = 0; i < FaceNormals.size(); i++)
    delete FaceNormals[i];
  FaceNormals.clear();
  for(i = 0; i < VertexNormals.size(); i++)
    delete VertexNormals[i];
  VertexNormals.clear();
  for(i = 0; i < FacesWithMaterials.size(); i++)
    delete FacesWithMaterials[i];
  FacesWithMaterials.clear();
  for(i = 0; i < Materials.size(); i++)
    delete Materials[i];
  Materials.clear();
  if(useRAPID) collisionModel.BeginModel(); // Zera a memoria na RAPID
}
// *******************************************************
vector<TVertices*> SmVR_COBJLoader::GetVertices() {
  return Vertices;
}
vector<TFaces*> SmVR_COBJLoader::GetFaces() {
  return Faces;
}
vector<TFacesMat*> SmVR_COBJLoader::GetFacesWithMaterials() {
  return FacesWithMaterials;
}
vector<TVertices*> SmVR_COBJLoader::GetFaceNormals() {
  return FaceNormals;
}
vector<TVertices*> SmVR_COBJLoader::GetVertexTextures() {
  return VertexTextures;
}
vector<TVertices*> SmVR_COBJLoader::GetVertexNormals() {
  return VertexNormals;
}
vector<TMaterial*> SmVR_COBJLoader::GetMaterials() {
  return Materials;
}
RAPID_model* SmVR_COBJLoader::GetCollisionModel() {
  return &collisionModel;
}
char* SmVR_COBJLoader::GetFileName() {
  return Name;
}
void SmVR_COBJLoader::GetBoundaries(SmVR_CPoint *min, SmVR_CPoint *max) {
  min->X = Min.x;
  min->Y = Min.y;
  min->Z = Min.z;
  max->X = Max.x;
  max->Y = Max.y;
  max->Z = Max.z;
}
// ***************************************************
void SmVR_COBJLoader::InvertNormalOrientation() {
  NormalOrientation = !NormalOrientation;
}
// ************************************************
// Returns the number of colliding points for this object
// ************************************************
int SmVR_COBJLoader::Collided(SmVR_CObjFromFile *obj, SmVR_Matrix mtx1, SmVR_Matrix mtx2, int mode) {
  double r1[3][3], r2[3][3], t1[3], t2[3];
  SmVR_RAPIDMatrixConversion(mtx1, r1, t1);
  SmVR_RAPIDMatrixConversion(mtx2, r2, t2);
  Collide(r1, t1, GetCollisionModel(), r2, t2, ((SmVR_COBJLoader*)obj)->GetCollisionModel(), (mode ? ALL_CONTACTS : FIRST_CONTACT));
  return RAPID_num_contacts;
}
// ***************************************************
int SmVR_COBJLoader::Load(char *filename) {
  char type[199], buff[2000], *buffptr, part1[200], part2[200];
  int linenum, coord, text, norm, cont, i;
  TFaces *Face;
  TVertices *vertice, vertice2;
  TFacesMat* facesMat;
  FILE *infile = fopen(filename, "r");

  if(infile == NULL) {
    printf("No INPUT file \"%s\"!!\n", filename);
    return 0;
  }
  sprintf(Name, "%s", filename);
  printf("Reading \"%s\"...\n", Name);
  if(Vertices.size()) {
    printf("Desalocando obj para recarregar\n");
    FreeObjectData();
  }
  for(linenum = 0; !feof(infile); linenum++) {
    ReadALine(infile, buff);  // read control line
    // Check to see if there is a material specification in the current line
    sscanf(buff, "%s%s", part1, part2);
    if(!strcmp(part1, "usemtl")) {
      if(FacesWithMaterials.size())
        FacesWithMaterials[FacesWithMaterials.size()-1]->LastFace = (Faces.size() ? Faces.size()-1 : 0);
      FacesWithMaterials.push_back(facesMat = new TFacesMat);
      facesMat->FirstFace = facesMat->LastFace = Faces.size();
      sprintf(facesMat->MaterialName, "%s", part2);
    } else if(!strcmp(part1, "mtllib")) {
      ReadMaterialFile(part2);
      part1[0] = part2[0] = '\0';
    }
    type[0] = 0;
    if(linenum % 1000 == 0) printf(".");
    vertice = new TVertices;
    sscanf(buff, "%s %f %f %f", &type, &vertice->x, &vertice->y, &vertice->z);
    // Read the faces
    if(toupper(type[0])=='F') { // we have a face
      char v[3][100];  // nnn/nnn/nnn   nnn/nnn/nnn  nnn/nnn/nnn

      buffptr = strchr(buff, ' ');  // skip over the "f " or "fo "
      Face = new TFaces;
      sscanf(buffptr, "%s %s %s", v[0], v[1], v[2]);
      for(i = 0; i < 3;i++) {
        cont = 0;
        while(1) { // count how many '/' we have in v[i]
          buffptr = strchr(v[i], '/');
          if(buffptr)
            *buffptr = ' ';
          else break;
          cont++;
        }
        coord = text = norm = -1;
        switch(cont) {
          case 0: coord = atoi(v[i]); break; // just the vertex
          case 1: sscanf(v[i], "%d %d", &coord, &text); break; // vertex and texture
          case 2: // the vertex, the texture and normal vector
            sscanf(v[i], "%d %d %d", &coord, &text, &norm);
            // Test is the face is like v//t
            if(norm < 0) { // This face has no texture info,
              norm = text; // but it has normals
              text = -1;
            }
            break;
          default: printf("Wrong number of '/' (%d) in face %d\n", cont, Faces.size());
        }
        Face->v[i] = coord-1;
        Face->texture[i] = text-1;
        Face->normal[i] = norm-1;
      }
      Faces.push_back(Face);
      FaceNormals.push_back(vertice);
      vertice2 = CalculateNormal(Vertices[Face->v[0]],
                Vertices[Face->v[1]], Vertices[Face->v[2]]);
      vertice->x = vertice2.x;
      vertice->y = vertice2.y;
      vertice->z = vertice2.z;
    } else if(toupper(type[0])=='V') { // can be VT, VN or just v
      if(toupper(type[1])=='N') { // Is this a VERTEX NORMAL ? (vn)
        VertexNormals.push_back(vertice);
      } else if(toupper(type[1])=='T') { // Is this a VERTEX TEXTURE? (vt)
        VertexTextures.push_back(vertice);
      } else if(toupper(type[0])=='V') { // it is just a regular vertice !
        Vertices.push_back(vertice);
        if(Vertices.size() > 1) {
          if(vertice->x > Max.x)
            Max.x = vertice->x;
          if(vertice->x < Min.x)
            Min.x = vertice->x;
          if(vertice->y > Max.y)
            Max.y = vertice->y;
          if(vertice->y < Min.y)
            Min.y = vertice->y;
          if(vertice->z > Max.z)
            Max.z = vertice->z;
          if(vertice->z < Min.z)
            Min.z = vertice->z;
        } else { // Inicializao
          Min.x = Max.x = vertice->x;
          Min.y = Max.y = vertice->y;
          Min.z = Max.z = vertice->z;
        }
      }
    }
  }
  fclose(infile);
  printf("\n");
  if(FacesWithMaterials.size()) // Set the last face for the last material
    FacesWithMaterials[FacesWithMaterials.size()-1]->LastFace = (Faces.size() ? Faces.size()-1 : 0);
  for(i = 0; i < FacesWithMaterials.size(); i++) // Search for material numbers
    FacesWithMaterials[i]->Mat = SearchForMaterialNumber(FacesWithMaterials[i]->MaterialName);
  if(useRAPID) 
  {
    printf ("Putting object faces in RAPID...\n" );
    RAPID_model_Read();
  }
  printf("Done !\n");
  return 1;
}
// ***************************************************
void SmVR_COBJLoader::Render() {
  int v, f, m, Mat;
  GLfloat White[] = {1,1,1,1};

  if(FacesWithMaterials.size() == 0) { // No material for this object
    glBegin(GL_TRIANGLES);
    for(f = 0; f < Faces.size(); f++) {
      if(!VertexNormals.size())
        glNormal3f(FaceNormals[f]->x, FaceNormals[f]->y, FaceNormals[f]->z);
      for(v = 0; v < 3; v++) { // get the vertices
        if(VertexTextures.size())
          glTexCoord2f(VertexTextures[Faces[f]->texture[v]]->x, VertexTextures[Faces[f]->texture[v]]->y);
        if(VertexNormals.size() > Faces[f]->normal[v])
          glNormal3f(VertexNormals[Faces[f]->normal[v]]->x, VertexNormals[Faces[f]->normal[v]]->y, VertexNormals[Faces[f]->normal[v]]->z);
        glVertex3f(Vertices[Faces[f]->v[v]]->x, Vertices[Faces[f]->v[v]]->y, Vertices[Faces[f]->v[v]]->z);
      }
    }
    glEnd();
  } else { // Display object using its material. Takes each material & draw faces associated to it
    glPushAttrib(GL_ALL_ATTRIB_BITS); // Save the current rendering status
    glFrontFace(NormalOrientation ? GL_CW : GL_CCW);
    glEnable(GL_COLOR_MATERIAL); // Ativa o "Color Tracking"
    //if(VertexTextures.size()) glDisable(GL_TEXTURE_2D); // Disable the use of texture
    for(m = 0; m < FacesWithMaterials.size(); m++) {
      Mat = FacesWithMaterials[m]->Mat;
      if(Mat >= 0) { // maybe there is a missing material
        // If there is a texture for this material...
        if(Materials[Mat]->TextureName[0] && Materials[Mat]->flag) {
          Materials[Mat]->TextObj.Use();
          glDisable(GL_COLOR_MATERIAL);
          glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, White);
          glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, White);
          glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, White);
        } else {
          GLfloat AmbienteColor[] = {Materials[Mat]->amb.r, Materials[Mat]->amb.g, Materials[Mat]->amb.b};
          GLfloat DifuseColor[] = {Materials[Mat]->dif.r, Materials[Mat]->dif.g, Materials[Mat]->dif.b, 1.0f };
          GLfloat EspecularColor[] = {Materials[Mat]->spe.r, Materials[Mat]->spe.g, Materials[Mat]->spe.b, 1.0f };
          glDisable(GL_TEXTURE_2D);
          glDisable(GL_COLOR_MATERIAL);
          glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, AmbienteColor);
          glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, DifuseColor);
          glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, EspecularColor);
        }
      }
      glBegin(GL_TRIANGLES);
      double p[3][3];

      for(f = FacesWithMaterials[m]->FirstFace; f <= FacesWithMaterials[m]->LastFace; f++) {
        if(!VertexNormals.size())
          glNormal3f(FaceNormals[f]->x, FaceNormals[f]->y, FaceNormals[f]->z);
        for(v = 0; v < 3; v++) { // get the vertices
          // If there is a texture for this material...
          if(Mat >= 0 && Materials[Mat]->TextureName[0])
            glTexCoord2f(VertexTextures[Faces[f]->texture[v]]->x, VertexTextures[Faces[f]->texture[v]]->y);
          if(VertexNormals.size() > Faces[f]->normal[v])
            glNormal3f(VertexNormals[Faces[f]->normal[v]]->x, VertexNormals[Faces[f]->normal[v]]->y, VertexNormals[Faces[f]->normal[v]]->z);
          glVertex3f(Vertices[Faces[f]->v[v]]->x, Vertices[Faces[f]->v[v]]->y, Vertices[Faces[f]->v[v]]->z);
          p[v][0] = Vertices[Faces[f]->v[v]]->x;
          p[v][1] = Vertices[Faces[f]->v[v]]->y;
          p[v][2] = Vertices[Faces[f]->v[v]]->z;
        }
      }
      glEnd();
    }
    glPopAttrib(); // Restore the old rendering status
  }
}
// *******************************************************
void SmVR_COBJLoader::recarregaTexturas() {
  for(int i = 0; i < Materials.size(); i++) {
    if(strlen(Materials[i]->TextureName)&& Materials[i]->flag) {
      printf("Reading Texture \"%s\"\n", Materials[i]->TextureName);
      Materials[i]->flag = Materials[i]->TextObj.Load(Materials[i]->TextureName);
    }
  }
}
// *******************************************************
void SmVR_COBJLoader::ImprimeObjeto(int v, int f, int fwm, int fn, int vt, int vn, int m) {
  int i, cont;
  if(v) {
    printf("Vertices:\n");
    for(i = 0; i < Vertices.size(); i++)
      printf("%d:%f,%f,%f\n", i, Vertices[i]->x, Vertices[i]->y, Vertices[i]->z);
  }
  if(f) {
    printf("Faces:\n");
    for(i = 0; i < Faces.size(); i++) {
      printf("%d:", i);
      for(cont = 0; cont < 3; cont++)
        printf("[v:%d,n:%d,t:%d]", Faces[i]->v[cont], Faces[i]->normal[cont], Faces[i]->texture[cont]);
      printf("\n");
    }
  }
  if(fwm) {
    printf("FacesWithMaterials:\n");
    for(i = 0; i < FacesWithMaterials.size(); i++)
      printf("material '%s'=%d,F:%d,L:%d\n",FacesWithMaterials[i]->MaterialName,FacesWithMaterials[i]->Mat,FacesWithMaterials[i]->FirstFace,FacesWithMaterials[i]->LastFace);
  }
  if(fn) {
    printf("Faces Normals:\n");
    for(i = 0; i < FaceNormals.size(); i++)
      printf("%d:%f,%f,%f\n", i, FaceNormals[i]->x, FaceNormals[i]->y, FaceNormals[i]->z);
  }
  if(vt) {
    printf("VertexTextures:\n");
    for(i = 0; i < VertexTextures.size(); i++)
      printf("%d:%f,%f,%f\n", i, VertexTextures[i]->x, VertexTextures[i]->y, VertexTextures[i]->z);
  }
  if(vn) {
    printf("VertexNormals:\n");
    for(i = 0; i < VertexNormals.size(); i++)
      printf("%d:%f,%f,%f\n", i, VertexNormals[i]->x, VertexNormals[i]->y, VertexNormals[i]->z);
  }
  if(m) {
    printf("Materiais:\n");
    for(i = 0; i < Materials.size(); i++) {
      printf("============ Material %d ============\n", i);
      printf("newmtl %s\n", Materials[i]->Name);
      printf("  Ns %6.3f\n", Materials[i]->Ns);
      printf("  d %6.3f\n", Materials[i]->d);
      printf("  illum %6.3f\n", Materials[i]->illum);
      printf("  Kd %6.3f %6.3f %6.3f\n", Materials[i]->dif.r, Materials[i]->dif.g, Materials[i]->dif.b);
      printf("  Ka %6.3f %6.3f %6.3f\n", Materials[i]->amb.r, Materials[i]->amb.g, Materials[i]->amb.b);
      printf("  Ks %6.3f %6.3f %6.3f\n", Materials[i]->spe.r, Materials[i]->spe.g, Materials[i]->spe.b);
      printf("  map_Kd %s=%d\n", Materials[i]->TextureName, Materials[i]->TextObj.texture[0]);
    }
  }
}
// *******************************************************
// Creates the RAPID structure
// *******************************************************
void SmVR_COBJLoader::RAPID_model_Read() {
  double p[3][3];
  collisionModel.BeginModel();
  for(int f = 0; f < Faces.size(); f++) {
    p[0][0] = Vertices[Faces[f]->v[0]]->x;
    p[0][1] = Vertices[Faces[f]->v[0]]->y;
    p[0][2] = Vertices[Faces[f]->v[0]]->z;
    p[1][0] = Vertices[Faces[f]->v[1]]->x;
    p[1][1] = Vertices[Faces[f]->v[1]]->y;
    p[1][2] = Vertices[Faces[f]->v[1]]->z;
    p[2][0] = Vertices[Faces[f]->v[2]]->x;
    p[2][1] = Vertices[Faces[f]->v[2]]->y;
    p[2][2] = Vertices[Faces[f]->v[2]]->z;
    collisionModel.AddTri(p[0], p[1], p[2], f);
  }
  collisionModel.EndModel();
}
// ******************************************************
//  Converts the SmallVR transformation matrix to RAPID Translation & Rotation Matrices
//  SmallVR uses a 4x4 transformation matrix
//  RAPID uses (a 1x3 Translation matrix),(a 3x3 Rotation matrix)
// ******************************************************
void SmVR_COBJLoader::SmVR_RAPIDMatrixConversion(SmVR_Matrix M_SmVR, double MR[3][3], double MT[3]) {
  int i, j;                                           // SVE matrix, rotation matrix, translation matrix
  for(i = 0; i < 3; i++) { // Copy the rotation part of the matrix
    for(j = 0; j < 3; j++) // Note that yhe matrix is transposed
      MR[i][j] = M_SmVR[j][i];
    MT[i] = M_SmVR[3][i]; // Copy the translation part of the matrix
  }
}
// *******************************************************
TVertices SmVR_COBJLoader::CalculateNormal(TVertices *v1, TVertices *v2, TVertices *v3) {
  TVertices va, vb, n;
  GLfloat modulo;
 // clculo do vetor Normal N=(v2-v1)x(v3-v1)
  va.x = v2->x - v1->x;  // vetor a va = (v2-v1)
  va.y = v2->y - v1->y;
  va.z = v2->z - v1->z;

  vb.x = v3->x - v1->x;  // vetor b vb = (v3-v1)
  vb.y = v3->y - v1->y;
  vb.z = v3->z - v1->z;

  n.x = (va.y * vb.z) - (va.z * vb.y);
  n.y = (va.z * vb.x) - (va.x * vb.z);
  n.z = (va.x * vb.y) - (va.y * vb.x);
  modulo = sqrt(n.x*n.x + n.y*n.y + n.z*n.z);
  n.x = n.x / modulo;
  n.y = n.y / modulo;
  n.z = n.z / modulo;
  return n;
}
// *******************************************************
//  Gets a line and clean it by removing leading white space, and other unusefull
//  The line is in 'char buff[2000]' vector
// *******************************************************
void SmVR_COBJLoader::ReadALine(FILE *infile, char buff[2000]) {
  int posi;
  fgets(buff, 1999, infile);
  // Kill nulls, tabs anything strange
  for(posi = 0; buff[posi] != 0x0d && buff[posi] != 0x0a; posi++) {
    if(buff[posi] < ' ') // anything instead of
      buff[posi] = ' ';  // printable things ?
  }
  //Kill all leading white space
  while(buff[0] == ' ' && buff[0] != 0x0d && buff[0] != 0x0a)
    memmove(&buff[0], &buff[1], 199);
  // Kill any space padding
  posi = 0;
  while(buff[posi] != 0x0d && buff[posi] != 0x0a) {
    // space padding
    if(buff[posi] == ' ' && buff[posi+1] == ' ')
      memmove(&buff[posi], &buff[posi+1], 1998-posi);
    else posi++;
  }
}
// *******************************************************
void SmVR_COBJLoader::TryToGetMaterialParameterStr(char *line, char *parameter, char *str) {
  char Part1[200];
  if(strstr(line, parameter)) {
    sscanf(line, "%s%s", Part1, str);
    if(strcmp(Part1, parameter)) {
      printf("Error parsing material property!\nLine read:%s\nPress ENTER.",line);
      getchar();
      return;
    }
    line[0] = '\0';
  }
}
// *******************************************************
// Get RGB parameter from material properties
// *******************************************************
void SmVR_COBJLoader::TryToGetMaterialParameterRGB(char *line, char *parameter, TRGB &color) {
  char Part1[200];
  TRGB temp;
  if(strstr(line, parameter)) {
    sscanf(line, "%s%f%f%f", Part1, &temp.r, &temp.g, &temp.b);
    if(strcmp(Part1, parameter)) {
      printf("Error parsing material property!\nPress ENTER.");
      getchar();
      return;
    }
    color = temp;
    line[0] = '\0';
  }
}
// *******************************************************
void SmVR_COBJLoader::TryToGetMaterialParameterFloat(char *line, char *parameter, float &value) {
  char Part1[200];
  if(strstr(line, parameter)) {
    sscanf(line, "%s%f", Part1, &value);
    if(strcmp(Part1, parameter)) {
      printf("Error parsing material property!\nPress ENTER.");
      getchar();
      return;
    }
    line[0] = '\0';
  }
}
// *******************************************************
void SmVR_COBJLoader::ProcessMaterial(FILE *MatFile, char *line) {
  char Part1[200], Part2[200];
  // check to see if this is really a material specification
  sscanf(line, "%s%s", Part1, Part2);
  if(strcmp(Part1, "newmtl")) { // if it is not a material specification
    printf("Error parsing material file !\nPress ENTER to exit.");
    getchar();
    return;
  }
  Materials.push_back(new TMaterial);
  Materials[Materials.size()-1]->flag = false;
  sprintf(Materials[Materials.size()-1]->TextureName, "");
  sprintf(Materials[Materials.size()-1]->Name, "%s", Part2);
  do { // read the material file until it is finished
    if(fgets(line, 200, MatFile) == NULL)
      return;
    if(strstr(line,"newmtl")) // restart the parsing of a new material
      ProcessMaterial(MatFile, line);
    // Process material properties
    TryToGetMaterialParameterStr(line, "map_Kd", Materials[Materials.size()-1]->TextureName);
    TryToGetMaterialParameterRGB(line, "Kd", Materials[Materials.size()-1]->dif);
    TryToGetMaterialParameterRGB(line, "Ka", Materials[Materials.size()-1]->amb);
    TryToGetMaterialParameterRGB(line, "Ks", Materials[Materials.size()-1]->spe);
    TryToGetMaterialParameterFloat(line, "Ns", Materials[Materials.size()-1]->Ns);
    TryToGetMaterialParameterFloat(line, "d", Materials[Materials.size()-1]->d);
    TryToGetMaterialParameterFloat(line, "illum", Materials[Materials.size()-1]->illum);
    // Seta o diretorio correto da textura
    if(strlen(Materials[Materials.size()-1]->TextureName)) {
      extract_file_path(Name, line);
      strcat(line, Materials[Materials.size()-1]->TextureName);
      strcpy(Materials[Materials.size()-1]->TextureName, line);
    }
  } while(!feof(MatFile));
}
// *******************************************************
void SmVR_COBJLoader::ReadMaterialFile(char *MaterialLibraryName) {
  char line[256];
  int i, j;
  FILE *MatFile;

  extract_file_path(Name, line);
  strcat(line, MaterialLibraryName);
  if((MatFile = fopen(line, "r"))== NULL) {
    printf("No MATERIAL file '%s' no obj '%s'!!\n", line, Name);
    return;
  }
  printf("Reading Material File \"%s\"\n", line);
  while(!feof(MatFile)) {
    fgets(line, 200, MatFile);
    if(line[0] == '#') continue;
    if(strstr(line, "newmtl")) ProcessMaterial(MatFile, line);
  }
  fclose(MatFile);
  // Load the Textures for the materials
  for(i = 0; i < Materials.size(); i++) {
    if(strlen(Materials[i]->TextureName)) { // if there is a texture file name
      bool flag = false;
      for(j = 0; j < i && !flag; ) {
        if(!strcmp(Materials[i]->TextureName, Materials[j]->TextureName)) {
          Materials[i]->TextObj = Materials[j]->TextObj;
          Materials[i]->flag = Materials[j]->flag;
          flag = !flag;
        }
        else j++;
      }
      if(!flag) {
        printf("Reading Texture \"%s\"\n", Materials[i]->TextureName);
        Materials[i]->flag = Materials[i]->TextObj.Load(Materials[i]->TextureName);
      }
    }
  }
}
// *******************************************************
// Given the Material name this function returns its number
// *******************************************************
int SmVR_COBJLoader::SearchForMaterialNumber(char *MaterialName) {
  for(int i = 0; i < Materials.size(); i++)
    if(!strcmp(Materials[i]->Name, MaterialName))
      return i;
  printf("Invalid Material name(%s) in object\n", MaterialName);
  return -1;
}
